@@ -140,13 +140,26 @@ Packet Subscriber::read_one(PubSubTopic topic, ReaderInit init, int flags) {
   auto cfo = c_fileopts(topic.file_opts);
   a0_pubsub_topic_t c_topic{topic.name.c_str(), &cfo};
 
-  auto data = std::make_shared<std::vector<uint8_t>>();
+  auto pkt_data = std::make_shared<std::vector<uint8_t>>();
+
+  // The alloc will be reused for (potentially) multiple packets, especially if blocking.
+  // We only need to keep the first one alive.
+  struct data_t {
+    std::vector<uint8_t>* pkt_data;
+    std::vector<uint8_t> dummy;
+  } data{pkt_data.get(), {}};
+
   a0_alloc_t alloc = {
-      .user_data = data.get(),
+      .user_data = &data,
       .alloc = [](void* user_data, size_t size, a0_buf_t* out) {
-        auto* data = (std::vector<uint8_t>*)user_data;
-        data->resize(size);
-        *out = {data->data(), size};
+        auto* data = (data_t*)user_data;
+        if (data->pkt_data->empty()) {
+          data->pkt_data->resize(size);
+          *out = {data->pkt_data->data(), size};
+        } else {
+          data->dummy.resize(size);
+          *out = {data->dummy.data(), size};
+        }
         return A0_OK;
       },
       .dealloc = nullptr,
@@ -154,7 +167,7 @@ Packet Subscriber::read_one(PubSubTopic topic, ReaderInit init, int flags) {
 
   a0_packet_t c_pkt;
   check(a0_subscriber_read_one(c_topic, alloc, init, flags, &c_pkt));
-  return Packet(c_pkt, [data](a0_packet_t*) {});
+  return Packet(c_pkt, [pkt_data](a0_packet_t*) {});
 }
 
 }  // namespace a0
